-- Asteroid Spotter Plugin
-- Visit http://web-adventures.org:3000/ for more information
-- Maintained by SlashGordon / Dentek Collector XX-62 in game. For use by WANG guild members only.

dofile("api.lua")

local api = SpotterAPI()

local spotter_on = false
local sector_roids = {}
local player_id 
local found = 0

function spotter_strsplit(pattern,text)
    local result = {}
    local theStart = 1
    local theSplitStart, theSplitEnd = string.find(text,pattern,theStart)
    while theSplitStart do
        table.insert(result,string.sub(text,theStart,theSplitStart-1))
        theStart = theSplitEnd + 1
        theSplitStart, theSplitEnd = string.find(text,pattern,theStart)
    end
    table.insert(result, string.sub(text,theStart))
    return result 
end
	
function spotter_scan(event, data)
    local objecttype,objectid = radar.GetRadarSelectionID()
	
    if(data ~= nil and data ~= "" and objecttype == 2) then
        local ores = {} 
        string.gsub(data,"(.-)\n", function(a) if(not string.find(a, "Temperature:")) then table.insert(ores, a) end end)
        if(#ores >= 1 and ores[1] ~= "Object too far to scan") then 
			local ore_string = ""
            for i,v in ipairs(ores) do 
                if v then
                    v = string.gsub(v, "\r", "")
                    v = string.gsub(v, "\n", "")
                    v = string.gsub(v, "%%", "")
                    v = string.gsub(v, " Ore", "")
                    local ore = spotter_strsplit(": ",v)
					if (ore[2] ~= nil) then ore_string = ore_string..ore[1]..": "..ore[2].." " end
                end
            end
			if (sector_roids[objectid] == nil) then
				print("\127f0f0f0".."<"..tostring(objectid).."> "..ore_string.."\127o")
				sector_roids[objectid] = ore_string
				api.postAsteroid(GetCurrentSectorid(), player_id, objectid, ore_string)
			end
        end
    end

    return objectid
end	
	
function spotter_target(event, data)
	local objecttype,objectid = radar.GetRadarSelectionID()
	if (objecttype == 2) then
		if (sector_roids[objectid] ~= nil) then
			print("\127f0f0f0".."<"..tostring(objectid).."> "..sector_roids[objectid].."\127o")
		else
			print("\127o<"..tostring(objectid).."> not scanned.\127o")
		end
	end
end	
	
	
function spotter_sector_changed(event, data)
	sector_roids = {}
	api.getAsteroidList(GetCurrentSectorid(), player_id, sector_roids)
end

	
function spotter_start()
	if spotter_on then
	   spotter_on = false
	   player_id = GetCharacterIDByName(GetPlayerName())
	   UnregisterEvent(spotter_scan, "TARGET_SCANNED")
	   UnregisterEvent(spotter_target, "TARGET_CHANGED")
	   UnregisterEvent(spotter_sector_changed, "SECTOR_CHANGED")
	   print('Spotter disabled.')
	else
	   spotter_on = true
	   RegisterEvent(spotter_scan, "TARGET_SCANNED")
	   RegisterEvent(spotter_target, "TARGET_CHANGED")
	   RegisterEvent(spotter_sector_changed, "SECTOR_CHANGED")
	   UnregisterEvent(spotter_start, "PLAYER_ENTERED_GAME")
	   print('Spotter enabled')
	end
end


function filter_asteroids_ss(ss, criteria)
	local range_roids = {}
	found = 0
	for i,v in pairs(sector_roids) do 
		local sv = string.lower(v)
		if (string.find(sv, ss) ~= nil) then
			if (criteria(i,v)) then                 
				local s,e = string.find(sv, ss)
				local sv0 = string.sub(sv, s)
				s,e = string.find(sv0, ": ")
				s = s + 2
				local sv1 = string.sub(sv0, s)
				local e1, ignore = string.find(sv1, " ")
				local purity = tonumber(string.sub(sv1, 1, e1))
				table.insert(range_roids, {i, purity})
				found = found + 1
			end
		end
	end
	return range_roids
end

function filter_asteroids(criteria)
	local range_roids = {}
	found = 0
	for i,v in pairs(sector_roids) do 
		if (criteria(i,v)) then                    
				table.insert(range_roids, {i, v})
				found = found + 1
		end
	end
	return range_roids
end
	
	
function spotter_find(data, args)
	-- Does this sector contain the ore ?
	local current_target = 0 
	local ss = string.lower(args[1])
	local objecttype,objectid = radar.GetRadarSelectionID()
	if (objecttype == 2) then current_target = objectid end

	-- don't want target to kick-in
	UnregisterEvent(spotter_target, "TARGET_CHANGED")
	
	local range_roids = filter_asteroids_ss(ss, function(i, v) 
		radar.SetRadarSelection(2, i)
		local objecttype,objectid = radar.GetRadarSelectionID()
		return ((objectid ~= current_target) and (i == objectid))
	end)

	if (found > 0) then
		table.sort(range_roids, function(a,b) 
						return(a[2] > b[2])
						end)
		RegisterEvent(spotter_target, "TARGET_CHANGED")
		radar.SetRadarSelection(2, range_roids[1][1])

	else
		-- nothing in radar range 
		-- find highest in sector
		local all_roids = filter_asteroids_ss(ss, function(i, v) return true end)
		table.sort(all_roids, function(a,b) 
			return(a[2] > b[2])
			end)
		
		if (found > 0) then
			
			-- We have a candidate
			local target_id = all_roids[1][1]

			print("\127f0f0f0"..'In sector: <'..tostring(target_id)..'> '..sector_roids[target_id].."\127o")
			
			-- find closest id in radar range
			range_roids = filter_asteroids(function(i, v) 
					radar.SetRadarSelection(2, i)
					local objecttype,objectid = radar.GetRadarSelectionID()
					return (i == objectid)
				end)
			table.sort(range_roids, function(a,b)
					local ad = math.abs(target_id - a[1])
					local bd = math.abs(target_id - b[1]) 
					return(ad < bd)
				end)
			-- target the closest
			RegisterEvent(spotter_target, "TARGET_CHANGED")

			radar.SetRadarSelection(2, range_roids[1][1])
			
		else
	    	-- Otherwise search nearby sectors
			-- reselect the old target
			RegisterEvent(spotter_target, "TARGET_CHANGED")
			radar.SetRadarSelection(objecttype, current_target)
			api.findAsteroid(GetCurrentSectorid(), player_id, args[1])
		end
	end
end

RegisterUserCommand('spotter', spotter_start)
RegisterUserCommand('find', spotter_find)

RegisterEvent(spotter_start, "PLAYER_ENTERED_GAME")

-- Commodities
--RegisterEvent(targetless.start, "LEAVING_STATION")
--RegisterEvent(targetless.stop, "ENTERING_STATION")
